use actix_web::{
    web::{Data, Payload},
    HttpResponse,
};
use sqlx::PgPool;

use flowy_document::protobuf::{QueryDocParams, SaveDocParams};
use flowy_net::errors::ServerError;

use crate::service::{
    doc_service::{read_doc, update_doc},
    util::parse_from_payload,
};

pub async fn read_handler(
    payload: Payload,
    pool: Data<PgPool>,
) -> Result<HttpResponse, ServerError> {
    let params: QueryDocParams = parse_from_payload(payload).await?;
    let response = read_doc(pool.get_ref(), params).await?;
    Ok(response.into())
}

pub async fn update_handler(
    payload: Payload,
    pool: Data<PgPool>,
) -> Result<HttpResponse, ServerError> {
    let params: SaveDocParams = parse_from_payload(payload).await?;
    let response = update_doc(pool.get_ref(), params).await?;
    Ok(response.into())
}
